% Author: Pieter Abbeel pabbeel@cs.berkeley.edu www.cs.berkeley.edu/~pabbeel
% 2009/11/07

% This file is not necessarily intended to be starter code for any of the questions.
% Its sole purpose is to illustrate some of the functions provided to you.
% In particular, this file illustrates 
% (a) how you can run an instantiation of
%     tetris for human play 
%   Controls (use numpad):
%    [4] Left  [5] Spin  [6] Right
%              [2] Drop
%    alternatively,
%    [Q] Left  [W] Spin  [E] Right
%              [S] Drop
%
%    [Ctrl+P] Pause/Unpause
% (b) the data you obtain stored in a set of global variables from human play: 
%        * chosen_map_log: a log containing all the board configurations that
%             were chosen during human play
%             chosen_map_log{k} will contain the k'th board configuration
%             encountered during play
%        * options_map_log: a log containing all the board configurations
%             that were available as a next state during human play
%             options_map_log{k}{i}: i indexes over all board situations
%             that were available to the player as next states; the one the
%             player chose is stored in chosen_map_log{k}
%         Note: the human player can choose actions not considered
%         available to the computer player.  I.e., the computer player is
%         supposed to choose a translatino and a rotation, and then simply
%         drop the block straight down.  As a consequence, it is possible
%         that the chosen_map_log{k} map is not equal to
%         options_map_log{k}{i} for any i.
%         Note2: when calling mtetris a 2nd, 3rd, etc. time, the new maps
%         will be appended to the back of the maps already stored in
%         options_map_log and chosen_map_log.
% (c) How you can have the computer play (currently playing randomly)

% start by running the following commands (it will ensure some calls will
% be executed as compiled c-code, rather than interpreted matlab code, this
% will speed up your runs)
mex tetris_place_block.c
mex tetris_standard_22_features.c
mex tetris_2_features.c

% play:
global options_map_log chosen_map_log map_idx % global variables recording tetris board situations, see above for details

mtetris % repeat as many games as you like

% now options_map_log and chosen_map_log will contain data

% let's compute the features for the chosen and options maps:
% [just to illustrate feature computation]
for i=1:length(chosen_map_log)
    chosen_phi_log{i} = tetris_standard_22_features(chosen_map_log{i});
    for j=1:size(options_map_log{i},2)
        options_phi_log{i}(:,j) = tetris_standard_22_features(options_map_log{i}{j});
    end
end



% let's have the computer play
board_data = init_board_data();

map = tetris_init_map(board_data);

[hmap, hfig] = tetris_init_draw(board_data); %inits drawing, we don't use hfig actually

game_over = 0;
max_i=1000;%max number of blocks we will let the computer play for
for i=1:max_i
    block_idx = ceil(rand*7);
    translation_action = ceil(rand*10); %choose from 1 to 10
    rotation_action = ceil(rand*4)-1; % choose from 0 to 3
    [map,game_over] = tetris_place_block(map, block_idx, translation_action, rotation_action, board_data);
    hmap = tetris_draw_now(hmap, map, board_data); %draws the current map; will slow things down, obviously
        %note: occasionally the "tetris_draw_now" fails and complains about an
    %invalide handle; not sure what the deal is; if you find out, let me
    %know; in the meantime, it really should not prevent you from cracking
    %you entire PS b/c most of the time you would not want to draw anyway
    %(b/c it takes time)
    if(game_over)
        break;
    end
    pause(.1); % allows us to watch it play (current policy is not all that clever and interesting to watch though!)
end
fprintf(1,['\n\nthe computer managed to place ' num2str(i) ' blocks onto the board\n\n']);



